from typing import Iterable
import numpy as np
import json
from collections import UserList

from asf_search.download import download_url


class ASFProduct:
    def __init__(self, args: dict):
        self.properties = args['properties']
        self.geometry = args['geometry']

    def __str__(self):
        return json.dumps(self.geojson(), indent=2, sort_keys=True)

    def geojson(self) -> dict:
        return {
            'type': 'Feature',
            'geometry': self.geometry,
            'properties': self.properties
        }

    def download(self, dir: str, filename: str = None, token: str = None) -> None:
        """
        Downloads this product to the specified path and optional filename.

        :param dir: The directory into which this product should be downloaded.
        :param filename: Optional filename to use instead of the original filename of this product.
        :param token: EDL authentication token for authenticated downloads, see https://urs.earthdata.nasa.gov/user_tokens

        :return: None
        """
        if filename is None:
            filename = self.properties['fileName']

        download_url(url=self.properties['url'], dir=dir, filename=filename, token=token)

    def stack(self) -> UserList:
        """
        Builds a baseline stack from this product.

        :return: ASFSearchResults(list) of the stack, with the addition of baseline values (temporal, perpendicular) attached to each ASFProduct.
        """
        from .search.baseline_search import stack_from_product

        return stack_from_product(self)

    def centroid(self) -> (Iterable[float]):
        """
        Finds the centroid of a product
        Shamelessly lifted from https://stackoverflow.com/a/23021198 and https://stackoverflow.com/a/57183264
        """
        arr = np.array(self.geometry['coordinates'][0])
        length, dim = arr.shape
        return [np.sum(arr[:, i]) / length for i in range(dim)]




from collections import UserList
import json

from asf_search.ASFProduct import ASFProduct


class ASFSearchResults(UserList):
    def geojson(self):
        return {
            'type': 'FeatureCollection',
            'features': [product.geojson() for product in self]
        }

    def __str__(self):
        return json.dumps(self.geojson(), indent=2, sort_keys=True)

    def download(self, dir: str, token: str = None) -> None:
        """
        Iterates over each ASFProduct and downloads them to the specified directory.

        :param dir: The directory into which the products should be downloaded.
        :param token: EDL authentication token for authenticated downloads, see https://urs.earthdata.nasa.gov/user_tokens

        :return: None
        """

        for product in self:
            product.download(dir=dir, token=token)



from importlib.metadata import PackageNotFoundError, version

from .ASFProduct import ASFProduct
from .ASFSearchResults import ASFSearchResults
from .exceptions import *
from .constants import *
from .health import *
from .search import *

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    print('package is not installed!\n'
          'Install in editable/develop mode via (from the top of this repo):\n'
          '   python -m pip install -e .\n'
          'Or, to just get the version number use:\n'
          '   python setup.py --version')


HH = 'HH'
VV = 'VV'
VV_VH = 'VV+VH'
HH_HV = 'HH+HV'
DUAL_HH = ''
DUAL_VV = ''
DUAL_HV = ''
DUAL_VH = ''
HH_3SCAN = ''
HH_4SCAN = ''
HH_5SCAN = ''
QUAD = 'quadrature'
HH_VV = 'HH+VV'
HH_HV_VH_VV = 'HH+HV+VH+VV'
FULL = 'full'
UNKNOWN = 'UNKNOWN'


import os.path
import urllib.parse
import requests
from importlib.metadata import PackageNotFoundError, version

from asf_search.exceptions import ASFDownloadError


def download_url(url: str, dir: str, filename: str = None, token: str = None) -> None:
    """
    Downloads a product from the specified URL to the specified location and (optional) filename.

    :param url: URL from which to download
    :param dir: Directory in which to save the product
    :param filename: Optional filename to be used, extracted from the URL by default
    :param token: EDL Auth Token for authenticating downloads, see https://urs.earthdata.nasa.gov/user_tokens
    :return:
    """
    if filename is None:
        filename = os.path.split(urllib.parse.urlparse(url).path)[1]

    if not os.path.isdir(dir):
        raise ASFDownloadError(f'Error downloading {url}: directory not found: {dir}')

    if os.path.isfile(os.path.join(dir, filename)):
        raise ASFDownloadError(f'File already exists: {os.path.join(dir, filename)}')

    try:
        pkg_version = version(__name__)
    except PackageNotFoundError:
        pkg_version = '0.0.0'
    headers = {'User-Agent': f'{__name__}.{pkg_version}'}
    if token is not None:
        headers['Authorization'] = f'Bearer {token}'

    response = requests.get(url, stream=True, allow_redirects=False)
    while 300 <= response.status_code <= 399:
        new_url = response.headers['location']
        if 'aws.amazon.com' in urllib.parse.urlparse(new_url).netloc:
            response = requests.get(new_url, stream=True, allow_redirects=False)  # S3 detests auth headers
        else:
            response = requests.get(new_url, stream=True, headers=headers, allow_redirects=False)
    response.raise_for_status()
    with open(os.path.join(dir, filename), 'wb') as f:
        for chunk in response.iter_content(chunk_size=8192):
            f.write(chunk)


HOST = 'api.daac.asf.alaska.edu'
HOST_TEST = 'api-test.asf.alaska.edu'
HOST_DEV = 'api-dev.asf.alaska.edu'
HOST_LOCAL = 'local.asf.alaska.edu:5000'

HEALTH_PATH = '/health'
SEARCH_PATH = '/services/search/param'
BASELINE_PATH = '/services/search/baseline'
MISSION_PATH = '/services/utils/mission_list'
DATE_PATH = '/services/utils/date'

CMR_HOST = 'cmr.earthdata.nasa.gov'
CMR_HOST_UAT = 'cmr.uat.earthdata.nasa.gov'
